import setuptools

with open("README.md", 'r') as f:
    long_description = f.read()

setuptools.setup(
    name = 'secretKeeper',
    version = '0.0.0',
    description = 'A simple password manager to be used with Good-Bot.',
    long_description = long_description,
    long_description_content_type="text/markdown",
    license = 'MIT',
    author = 'Tricky',
    packages=setuptools.find_packages(),
    entry_points = {
        'console_scripts': [
            'secretKeeper = secretKeeper.__main__:main'
            ]
        }
)



from cryptography.fernet import Fernet
import filecmp
import shutil
import tempfile
import argparse

# ---------------------------------------------------------------------
#                           Key Creation
# ---------------------------------------------------------------------
def create_key(path):
    '''

    This function creates a key to encrypt and 
    decrypt files. Keep this key safely. If you 
    lose this key, your file containing your 
    passwords is lost forever!

    path (click.File): An opened file that we can write to.

    returns: True

    creates: A file that contains the key for later use.

    '''

    key = Fernet.generate_key()

    path.write(key)

    return True
     
# ---------------------------------------------------------------------
#                        New Passwords Files
# ---------------------------------------------------------------------
def create_password(passwords, key, newfile):
    '''
    This functions encrypts every passwords in the provided list using 
    the provided key. It then writes the passwords to newfile.

    passwords (list of strings): A list of unencrypted passwords.

    key (str): The Fernet key that will be used to encrypt the 
    passwords.

    newfile (click.File): An opened file that we can write to.
    '''
    f = Fernet(key)
    for password in passwords:
        encoded = password.encode('utf-8')
        encrypted = f.encrypt(encoded)
        newline = '\n'.encode('utf-8')
        text = encrypted + newline
        newfile.write(text)

    return newfile.name

# ---------------------------------------------------------------------
#                   Adding To Existing Password File
# ---------------------------------------------------------------------

def adder(password, key, existing_file, position):
    '''
    This function encrypts password using key. It then adds password to
    existing file at the provided position.

    password (str): The password to add.

    key (str): A Fernet key (The one that was used to create the 
    file).

    existing_file (click.File): An opened file that we can write to.

    position (int): Where to write the new password assuming that the 
    existing passwords are in a list splitted on newlines.

    returns (str): P
    '''
    f = Fernet(key)
    lines = existing_file.readlines()

    # Encrypting the password.
    encoded = password.encode('utf-8')
    encrypted = f.encrypt(encoded)
    newline = '\n'.encode('utf-8')
    to_add = encrypted + newline

    # Adding the password to the list.
    lines[position:position] = [to_add]
    
    # Creating temporary file and writing to it.
    x = tempfile.NamedTemporaryFile(mode = 'w+b', delete = False)
    x.writelines(lines)
    x.close()

    # Copying the file to the new file
    shutil.move(x.name, existing_file.name) 
    
    return existing_file.name



import click
from funcmodule import *

@click.group()
def main():
    '''
    This is a short encryption tool that can be used to 
    generate Fernet encryption keys and encrypt passwords.
    '''
    pass

# ---------------------------------------------------------------------
#                           Key Creation
# ---------------------------------------------------------------------

# Creating a new command.
@click.command()

# Adding possible arguments.
@click.argument(
        'output', 
        type = click.File('wb')
        )
def keygen(output):
    '''
    Generates a Fernet key and writes the key to the
    specified output file.
    '''
    create_key(output)
    message = '''\
            Your fernet key has been written under %s. Keep it safe!
            '''%(output.name)
    click.echo(message)
    
    return None

# ---------------------------------------------------------------------
#                        New Passwords Files
# ---------------------------------------------------------------------

# Creating a new command.
@click.command()

# Adding possible arguments.
@click.argument(
        'output',
        type = click.File('wb')
        )
@click.argument(
        'key',
        type=click.File('rb')
        )
@click.option(
        '-n',
        '--number', 
        default=1, 
        show_default = True, 
        type = int,
        help = '''\
                The number of passwords you want to write to the file.
                '''
        )
def new_passwords(output, key, number):
    '''
    Encrypt passwords provided by the user using a provided key. The 
    encrypted passwords are then written to a new file.
    '''
    passwords = []
    for i in range(number):
        to_prompt = 'Password #%s'%(i+1)
        passwords.append(click.prompt(to_prompt, hide_input = True))

    key_name = key.name
    key = key.read()

    name = create_password(passwords, key, output)

    message = '''\
            Your passwords have been saved in %s using %s.
            '''%(name, key_name)

    click.echo(message)
    return None
# ---------------------------------------------------------------------
#                   Adding To Existing Password File
# ---------------------------------------------------------------------

# Creating a new command.
@click.command()

# Adding possible arguments.
@click.argument(
        'existing_file',
        type = click.File('rb')
        )
@click.argument(
        'key',
        type = click.File('rb')
        )
@click.option(
        '-p',
        '--position',
        default = -1,
        show_default = True,
        type = int,
        help = '''\
                Assuming your passwords are stored as a list, this is 
                the index where the new password will be added.'''
        )
def add_password(existing_file, key, position):
    '''
    Adds a new password to the provided file. If a position is 
    provided, the password will be inserted at this index. If password 
    0,1,2,3, and new password is inserted at index = 2, password 2 
    becomes password 3, and 3 becomes password 4.
    '''
    key = key.read() 

    new_password =  click.prompt('New password', hide_input = True)

    location = adder(new_password, key, existing_file, position)

    message = '''\
            Your password has been saved in %s at position %s.
            '''%(location, position)
    
    click.echo(message)

    return None
     

# Adding the commands to the main click object.
main.add_command(add_password)
main.add_command(keygen)
main.add_command(new_passwords)

if __name__ == '__main__':
    main()
